/* alephextra.h: banner etc. for Aleph.

   This is included by Aleph, from ../alephextra.c
   (generated from ../lib/texmfmp.c).

This file is part of Aleph.

Copyleft (c) 2004 the Aleph team

Aleph is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Aleph is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Aleph; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

*/

#define BANNER "This is Aleph, Version 3.141592--1.15--2.1 (RC2)"
#define COPYRIGHT_HOLDER "the Aleph task force"
#define AUTHOR NULL
#define PROGRAM_HELP ALEPHHELP
#define DUMP_VAR TEXformatdefault
#define DUMP_LENGTH_VAR formatdefaultlength
#define DUMP_OPTION "afmt"
#define DUMP_EXT ".afmt"
#define INPUT_FORMAT kpse_tex_format
#define INI_PROGRAM "inialeph"
#define VIR_PROGRAM "viraleph"
